# frozen_string_literal: true

require 'nokogiri'
require_relative 'wordpress_xml/post'
require_relative 'wordpress_xml/attachment'

module SuttyMigration
  # Understands the XML dump generated by Wordpress and creates
  # Jekyll::Documents
  class WordpressXml
    attr_reader :site, :file, :xml

    # @param :site [Jekyll::Site] Jekyll site
    # @param :file [String] File path
    def initialize(site:, file:)
      @site = site
      @file = file
      @xml  = Nokogiri::XML File.read(file)

      # Make things easier by removing namespaces.
      xml.remove_namespaces!
    end

    def inspect
      '#<SuttyMigration::WordpressXml>'
    end

    # Site URL
    #
    # @return [String]
    def url
      @url ||= attribute_value(xml, 'channel > link')
    end

    # Site title
    #
    # @return [String]
    def title
      @title ||= attribute_value(xml, 'channel > title')
    end

    # Description
    #
    # @return [String]
    def description
      @description ||= attribute_value(xml, 'channel > description')
    end

    # Language
    #
    # TODO: Migrate multilanguage sites.
    #
    # @return [String]
    def language
      @language ||= attribute_value(xml, 'channel > language')
    end

    # Authors with attributes, indexed by author email.
    #
    # @return [Hash]
    def authors
      @authors ||= xml.css('channel > author').map do |author|
        {
          attribute_value(author, 'author_email') => {
            id: attribute_value(author, 'author_id').to_i,
            display_name: attribute_value(author, 'author_display_name'),
            first_name: attribute_value(author, 'author_first_name'),
            last_name: attribute_value(author, 'author_last_name'),
            email: attribute_value(author, 'author_email')

          }
        }
      end.reduce(&:merge)
    end

    # Categories with attributes, indexed by slug ("nicename")
    #
    # @return [Hash]
    def categories
      @categories ||= xml.css('channel > category').map do |category|
        {
          attribute_value(category, 'category_nicename') => {
            id: attribute_value(category, 'term_id').to_i,
            title: attribute_value(category, 'cat_name'),
            parent: attribute_value(category, 'category_parent'),
            slug: attribute_value(category, 'category_nicename')
          }
        }
      end.reduce(&:merge)
    end

    # Tags with attributes, indexed by slug
    #
    # @return [Hash]
    def tags
      @tags ||= xml.css('channel > tag').map do |tag|
        {
          attribute_value(tag, 'tag_slug') => {
            id: attribute_value(tag, 'term_id').to_i,
            title: attribute_value(tag, 'tag_name'),
            slug: attribute_value(tag, 'tag_slug')
          }
        }
      end.reduce(&:merge)
    end

    # Posts, indexed by ID
    #
    # @return [Hash]
    def posts
      @posts ||= items_find_by('post_type', 'post').map do |post|
        { attribute_value(post, 'post_id').to_i => Post.new(wordpress: self, item: post) }
      end.reduce(&:merge)
    end

    # Pages, indexed by ID
    #
    # @return [Hash]
    def pages
      @pages ||= items_find_by('post_type', 'page').map do |page|
        { attribute_value(page, 'post_id').to_i => Post.new(wordpress: self, item: page) }
      end.reduce(&:merge)
    end

    # Attachments, indexed by ID
    #
    # @return [Hash]
    def attachments
      @attachments ||= items_find_by('post_type', 'attachment').map do |attachment|
        { attribute_value(attachment, 'post_id').to_i => Attachment.new(wordpress: self, item: attachment) }
      end.reduce(&:merge)
    end

    # Find items by attribute and value
    #
    # @param [String] Attribute name
    # @param [String] Attribute value
    # @return [Nokogiri::NodeSet]
    def items_find_by(attribute, value)
      xml.css('channel > item').select do |item|
        attribute_value(item, attribute) == value
      end
    end

    # Get element's attribute value
    #
    # @param [Nokogiri::XML::Element]
    # @param [String]
    # @return [String]
    def attribute_value(element, attribute)
      element.at_css(attribute).text
    end
  end
end
